require "import"
import "android.widget.*"
import "android.view.*"
import "android.content.Context"
import "android.speech.tts.TextToSpeech"
import "java.util.Locale"

-- Initialize TextToSpeech engine
local tts
tts = TextToSpeech(activity, TextToSpeech.OnInitListener {
    onInit = function(status)
        if status == TextToSpeech.SUCCESS then
            tts.setLanguage(Locale.getDefault())
        end
    end
})

-- Function to speak the given text
local function speakText(text)
    if text ~= nil then
        tts.speak(tostring(text), TextToSpeech.QUEUE_FLUSH, nil, nil)
    end
end

-- Set title for the calculator
activity.setTitle("Welcome to this Calculator. Created by Sushil Rathore")

-- Create a layout for the calculator
local layout = LinearLayout(activity)
layout.setOrientation(LinearLayout.VERTICAL)

-- Create a TextView to display input and result
local output = TextView(activity)
output.setTextSize(30)
layout.addView(output)

-- Create buttons for numbers and operators, including 'Clear'
local buttons = {
    {'Clear', 'Delete', '/', '7', '8'},
    {'9', '*', '4', '5', '6'},
    {'+', '-', '1', '2', '3'},
    {'%', '.', '0', '=', ','}
}

-- Keep track of the last character for deletion announcement
local lastCharDeleted = nil

-- Create buttons and set their onClick listeners
for i = 1, #buttons do
    local row = buttons[i]
    local buttonLayout = LinearLayout(activity)
    buttonLayout.setOrientation(LinearLayout.HORIZONTAL)

    for j = 1, #row do
        local buttonText = row[j]
        local button = Button(activity)
        button.setText(buttonText)
        button.setOnClickListener(View.OnClickListener {
            onClick = function(view)
                if buttonText == 'Clear' then
                    lastCharDeleted = output.getText()
                    output.setText('')
                    speakText("Cleared. The text was: " .. tostring(lastCharDeleted))
                elseif buttonText == 'Delete' then
                    lastCharDeleted = output.getText():sub(-1)
                    local currentText = output.getText()
                    if currentText ~= nil and #currentText > 0 then
                        output.setText(currentText:sub(1, #currentText - 1))
                        speakText("Deleted digit: " .. tostring(lastCharDeleted))
                    end
                elseif buttonText == '=' then
                    -- Calculate result
                    local expression = output.getText()
                    if expression ~= nil and expression ~= '' then
                        -- Handle percentage calculation
                        expression = expression:gsub('%%', '*0.01*')
                        local result = load("return " .. expression)()
                        output.setText(tostring(result))
                        speakText("The result of " .. expression .. " is " .. tostring(result))
                    end
                else
                    output.setText(output.getText() .. buttonText)
                    speakText(buttonText)
                end
            end
        })
        buttonLayout.addView(button)
    end

    layout.addView(buttonLayout)
end

activity.setContentView(layout)